""" Vindby
    Copyright (C) 2018  Esther Dornhelm

    This program is licensed under an Attribution-NonCommercial 4.0 International (CC BY-NC 4.0) license

    Terms and conditions of this license are defined in
    file 'LICENSE.txt', which is part of this source code package.

    See <https://creativecommons.org/licenses/by-nc/4.0/>."""

from tkinter import *
import threading, math, timeit
from functools import partial

class App(threading.Thread):
    """Open Interface"""

    def __init__(self, game):
        threading.Thread.__init__(self)
        self.start()
        self.g = game
        self.sea = None
        self.clock = None
        self.photos = {}
        self.labels = {}
        self.ind = False
        self.cellButtons=[]
        self.infopanel=None

    def callback(self):
        self.root.destroy()
        import sys
        sys.exit(0)

    def displayCellInfo(self,cell):
        self.labels['cellinfo'].configure(text="Click on a cell to review cell and wind farm information.")

    def run(self):
        self.root = Tk()
        self.root.protocol("WM_DELETE_WINDOW", self.callback)
        Grid.rowconfigure(self.root, 1, weight=1)
        Grid.columnconfigure(self.root, 1, weight=1)
        Grid.rowconfigure(self.root, 0, weight=1)
        Grid.columnconfigure(self.root, 2, weight=1)
        Grid.rowconfigure(self.root, 0, weight=1)
        Grid.columnconfigure(self.root, 1, weight=1)
        Grid.columnconfigure(self.root, 3, weight=1)
        land = Frame(self.root, bg="#006400", width=100, height=20)
        land.grid(row=0, column=1, sticky=N+E + W+S)
        Grid.rowconfigure(land, 0, weight=0)
        Grid.columnconfigure(land, 0, weight=0)

        self.sea = Frame(self.root, bg="blue", width=100, height=100)
        self.sea.grid(row=1, column=1, sticky=N + S + E + W)

        self.infopanel=Frame(self.root, bg="white", width=50, height=60, borderwidth=5,relief=GROOVE)
        self.infopanel.grid(row=0, rowspan=5, column=2, sticky=N+W+S+E)

        self.cellinfopanel = Frame(self.root, bg="white", width=50, height=60, borderwidth=5, relief=GROOVE)
        self.cellinfopanel.grid(row=0, rowspan=5, column=0, sticky=N+W+S+E)

        self.labels['clickcell'] = Label(self.cellinfopanel, text="Click on a cell for cell and farm info:",bg="white", fg="black")
        self.labels['clickcell'].grid(row=0, column=0, sticky=N + W)

        self.labels['cellinfo'] = Label(self.cellinfopanel,text="Cell:\nMean Wind Speed [m/s]:\nWater Depth [m]:\nSoil Quality:\nDistance to Shore [km]:\nEnvironmental Restrictions:\t\nVessel Route:\nStatus:",bg="white", fg="black",justify=LEFT)
        self.labels['cellinfo'].grid(row=1,column=0,sticky=W)

        self.labels['farminfo'] = Label(self.cellinfopanel,text="Farm:\nAge [yrs]:\nInstalled Capacity:\nNumber of Turbines:\nFoundation:\nO&M Strategy:\nCapacity Factor:\nRevenue to Date:\nStatus:\nCapital Cost:\nO&M Cost:\nLCOE:\nFailure/turbine/yr:",bg="white", fg="black",justify=LEFT)
        self.labels['farminfo'].grid(row=2,rowspan=3,column=0,sticky=W)

        self.photos['substation'] = PhotoImage(file='icons/substation.png').subsample(16)
        self.labels['substation'] = Label(land, image=self.photos['substation'])
        self.labels['substation'].grid(row=0, column=0, sticky=W,padx=5,pady=4)

        self.bottom=Frame(self.root, bg="white", width=90, borderwidth=5,relief=GROOVE)
        self.bottom.grid(row=6,column=0,columnspan=3)

        self.bottom2 = Frame(self.root, bg="white", width=90, borderwidth=5, relief=GROOVE)
        self.bottom2.grid(row=7, column=0, columnspan=3)

        self.labels['Fun Facts'] = Label(self.bottom, width=90)
        self.labels['Fun Facts'].grid(row=0,column=0,sticky=N)
        self.labels['Fun Facts'].configure(text='Welcome to Vindby!')

        i=0
        self.labels['clock'] = Label(self.infopanel, text=self.g.tool.convertTime(self.g.gameClock)['Date'], bg="white", fg="black")
        self.labels['clock'].grid(row=i, column=0)
        i+=1
        self.labels['Balance']=Label(self.infopanel, text="Balance: ")
        self.labels['Balance'].grid(row=i,column=0,sticky=N+W)
        i += 1
        self.labels['CapitalCosts']=Label(self.infopanel, text='Capital costs:')
        self.labels['CapitalCosts'].grid(row=i, column=0, sticky=N + W)
        i += 1
        self.labels['OperatingCosts'] = Label(self.infopanel, text='Operating costs:')
        self.labels['OperatingCosts'].grid(row=i, column=0, sticky=N + W)
        i += 1
        self.labels['Revenue'] = Label(self.infopanel, text='Revenue to date:')
        self.labels['Revenue'].grid(row=i, column=0, sticky=N + W)
        i += 1
        self.labels['Capacity'] = Label(self.infopanel, text="Installed capacity: ")
        self.labels['Capacity'].grid(row=i, column=0, sticky=N + W)
        i += 1
        self.labels['Homes'] = Label(self.infopanel, text="Homes powered: \t\t")
        self.labels['Homes'].grid(row=i, column=0, sticky=N+W)
        i += 1
        self.labels['renewablePercent']=Label(self.infopanel, text="Energy share: ")
        self.labels['renewablePercent'].grid(row=i, column=0, sticky=N + W)
        i += 1
        self.labels['Stakeholders'] = Label(self.infopanel, text="Stakeholder brownie points:\n\tPublic:\n\tGovernment:\n\tShareholders:",justify=LEFT)
        self.labels['Stakeholders'].grid(row=i, column=0, sticky=W)
        i += 1
        self.labels['Scores'] = Label(self.infopanel,text="\n\t\tScores: \nProfitability:\t\t\nClimate:\t\t\nStakeholders\t\t\nCapacity Factor:\t\t\nTotal Score:\t\t",justify=LEFT)
        self.labels['Scores'].grid(row=i, column=0, sticky=N + W)

        for cell in range(len(self.g.seaGrid.cellList)):
            c = self.g.seaGrid.cellList[cell]
            lat = int(c.ycoor)
            long = int(c.xcoor)
            Grid.rowconfigure(self.sea, lat, weight=1)
            Grid.columnconfigure(self.sea, long, weight=1)
            B = Button(self.sea, text=cell + 1, bg='#4682B4', fg='white', width=2, height=1, command=partial(self.displayInfo,c))
            B.grid(row=lat, column=long, sticky=N + S + E + W)
            self.cellButtons.append(B)
        self.update()
        self.ind = True
        self.root.mainloop()

    def updateCellColor(self,cell):
        if cell.status == 'occupied' and cell.myFarm.status['phase'] == 'Operating' and cell.myFarm.farmFailure == {}:
            cell_color = 'green'
            font_color = 'black'
        elif cell.status == 'occupied' and cell.myFarm.status['phase'] == 'Under Construction' and cell.myFarm.farmFailure == {}:
            cell_color = 'yellow'
            font_color = 'black'
        elif cell.status == 'unoccupied' and cell in self.g.interact.investigatedCells:
            cell_color = 'cyan'
            font_color = 'black'
        elif cell.status == 'occupied' and cell.myFarm.farmFailure != {}:
            cell_color = 'red'
            font_color = 'white'
        elif cell.status == 'Decommissioned':
            cell_color = '#6495ED'
            font_color = 'white'
        else:
            cell_color = '#4682B4'
            font_color = 'white'
        self.cellButtons[cell.id-1].configure(bg=cell_color, fg=font_color)

    def updateCellName(self,cell):
        self.cellButtons[cell.id-1].configure(text=str(cell+1) + ' ' + str(cell.myFarm.name))
        print('text updated')

    def roundTo(self,value,nearest):
        return int(math.ceil(value / nearest)) * nearest

    def update(self):
        self.labels['clock'].configure(text=self.g.tool.convertTime(self.g.windClock)['Date'])
        goal_items = [(self.g.wallet.profits - self.g.wallet.operatingCosts) / self.g.wallet.initialInvestment,
                      self.g.tool.scores['Total'],
                      self.g.energyDemand.renewableShare * 100,
                      self.g.co2.temp,
                      0]
        n = self.g.myGoal['index']
        goal_item = goal_items[n]
        paused = " # # # # # # # # # # # # # # # # # # # # GAME IS PAUSED # # # # # # # # # # # # # # # # # # # # " if self.g.paused == True else ""
        self.labels['Fun Facts'].configure(text='Your goal: {}\nYear: {}, {} : {:3.3f} \n {}'.format(
                                           self.g.myGoal['Description'],
                                           int(self.g.tool.convertTime(self.g.windClock)['calendar year']),
                                           self.g.myGoal['measure'], goal_item,paused))

        self.labels['Balance'].configure(text='Balance: \t\t€{:,}'.format(self.roundTo(self.g.wallet.balance,100000)))
        self.labels['CapitalCosts'].configure(text='Capital costs: \t€{:,}'.format(self.roundTo(self.g.wallet.capitalCosts,100000)))
        self.labels['OperatingCosts'].configure(text='Operating costs: \t€{:,}'.format(self.roundTo(self.g.wallet.operatingCosts, 100000)))
        self.labels['Revenue'].configure(text='Revenue to date: \t€{:,}'.format(self.roundTo(self.g.wallet.profits,100000)))
        self.labels['Capacity'].configure(text='Installed capacity: {:,} MW'.format(self.g.windFarmGrid.installedCapacity))
        self.labels['Homes'].configure(text='Homes powered : {:,}'.format(self.roundTo(self.g.co2.homes,100)))
        self.labels['renewablePercent'].configure(text='Energy share: {:2.3f}% '.format(self.g.energyDemand.renewableShare * 100))
        self.labels['Stakeholders'].configure(text="Stakeholder brownie points:\n\tPublic: {}\n\tGovernment: {}\n\tShareholders: {}".format(self.g.public.satisfaction,self.g.govt.satisfaction,self.g.shareholders.satisfaction),justify=LEFT)
        self.labels['Scores'].configure(text="\n\tScores: \n"
                                             "Profitability:\t\t{:2.1f}/25 \n"
                                             "Climate:\t\t\t{:2.1f}/25 \n"
                                             "Stakeholders\t\t{:2.1f}/25 \n"
                                             "Capacity Factor:\t\t{:2.1f}/25 \n"
                                             "Total Score:\t\t{:2.1f}/100".format(self.g.tool.scores['Profitability'],
                                                                                  self.g.tool.scores['Climate Impact'],
                                                                                  self.g.tool.scores['Stakeholders'],
                                                                                  self.g.tool.scores['Capacity Factor'],
                                                                                  self.g.tool.scores['Total'],justify=LEFT))

    def displayInfo(self,c):
        if c in self.g.interact.investigatedCells:
            self.labels['cellinfo'].configure(text="Cell: " + str(c.id) + "\nMean Wind Speed [m/s]: " + str(c.meanU90) +
                                                   "\nWater Depth [m]: " + str(c.depth) + "\nSoil Quality: " + str(c.soilqual) +
                                                   "\nDistance to Shore [km]: " + str(c.dist2shore) +
                                                   "\nEnvironmental Restrictions: " + str(c.env_protection) +
                                                   "\nVessel Route: " + str(c.shipping_channel) + "\nStatus: " + str(c.status))
        else:
            message = "Need Investigation"
            self.labels['cellinfo'].configure(text="Cell: " + str(c.id) + "\nMean Wind Speed [m/s]: " + message +
                                                   "\nWater Depth [m]: " + str(c.depth) + "\nSoil Quality: " + message +
                                                   "\nDistance to Shore [km]: " + str(c.dist2shore) +
                                                   "\nEnvironmental Restrictions: " + message +
                                                   "\nVessel Route: " + message + "\nStatus: " + str(c.status))

        if c.status == 'occupied':
            self.labels['farminfo'].configure(text="Farm: " + str(c.myFarm.name) + "\nAge [yrs]: " + str(round(c.myFarm.status['age']/525600,1)) + "\nInstalled Capacity: " + str(c.myFarm.installedCapacity) +
                                                   "MW\nNumber of Turbines: " + str(len(c.myFarm.myTurbineList)) + '\nFoundation: ' + str(c.myFarm.mySubstructureList[-1].type.type) + "\nO&M Strategy: " + str(c.myFarm.OandM.strategy) +
                                                   "\nCapacity Factor: " + str(c.myFarm.capacityFactor) + "\nRevenue to Date: €" + str('{:,}'.format(self.roundTo(c.myFarm.revenue,100000))) +
                                                   "\nStatus: " + str(c.myFarm.status['phase']) + "\nCapital Cost: €" + str('{:,}'.format(self.roundTo(c.myFarm.costs['CAPEX'],100000))) +
                                                   "\nO&M Cost: €" + str('{:,}'.format(self.roundTo(c.myFarm.costs['OPEX'],100000))) +
                                                   "\nSelling Price: €" + str(int(c.myFarm.sellingPrice['First10Years'])) + "/MWh\nLCOE: €" +
                                                   str(c.myFarm.LCOE.displayValue)+'/MWh' + '\nFailure/turbine/yr: ' + str(c.myFarm.OandM.failuresPerTurbinePerYear))
        if c.status == 'unoccupied':
            self.labels['farminfo'].configure(text="Farm:\nAge [yrs]:\nInstalled Capacity:\nNumber of Turbines:\nFoundation:"
                                                   "\nO&M Strategy:\nCapacity Factor:\nRevenue to Date:"
                                                   "\nStatus:\nCapital Cost:\nO&M Cost:\nLCOE:\nFailure/turbine/yr:")
