""" Vindby
    Copyright (C) 2018  Esther Dornhelm

    This program is licensed under an Attribution-NonCommercial 4.0 International (CC BY-NC 4.0) license

    Terms and conditions of this license are defined in
    file 'LICENSE.txt', which is part of this source code package.

    See <https://creativecommons.org/licenses/by-nc/4.0/>."""

class FailureRates:
    """Represents default failure rates"""

    def __init__(self):
        self.failureRates={}
        self.turbineFailureRates={}
        self.structureFailureRates={}
        self.farmFailureRates={}
        self.ConstFailureRates={}
        self.otherfailures={}
        self.preventableFailures = []

    def read(self):                 #Initialize starting values
        f=open('tfailurerates.txt','r')
        self.failureRates={}
        for line in f:
            line = line.strip()
            columns = line.split()
            failuretype=columns[0]   #Type of failure
            self.failureRates[str(failuretype)]=float(columns[1])
        f.close()
        for failuretype,rate in self.failureRates.items(): #Document which failure types can be prevented with condition based monitoring, if the user chooses.
            if 'turbine' in failuretype or 'structure_direct' in failuretype:
                self.preventableFailures.append(failuretype)

    def sort(self):                 #Sort failures by type from input file : farm-wide faiures, turbine failures, and support structure failures
        self.turbineFailureRates = {}
        self.farmFailureRates = {}
        for failure, rate in self.failureRates.items():
            if 'turbine' in failure:
                self.turbineFailureRates[failure]=float(rate)
            elif 'structure' in failure:
                self.structureFailureRates[failure] = float(rate)
            elif 'farm' in failure:
                self.farmFailureRates[failure] = float(rate)
            elif 'construction' in failure:
                self.ConstFailureRates[failure] = float(rate)
            elif 'conditionbasedmonitoring' in failure:
                self.otherfailures[failure] = float(rate)

    def update(self,failure,change):        #Update failure rates based on actions in the game
        old=self.failureRates[failure]
        if self.failureRates[failure] != 0: #If we are modified a rate, doing so with multiplication factor
            self.failureRates[failure] = old*change
        else:                               #If the failure rat is otherwise zero initially
            self.failureRates[failure] = change
        self.sort()                         #Resort to update turbine and farm failure lists without undoing all previous updates made to the master list in failureRates

    # def seasonality(self):
    #     self.turbineFailureRates_HighWindSpeed = {}
    #     for failure, rate in self.turbineFailureRates.items():
    #         self.turbineFailureRates_HighWindSpeed[failure] = rate * self.failureRates['mechanical_failures_high_windspeeds']



# x=FailureRates()
# x.read()
# x.sort()
# x.update('turbine_mechanical_major',5)

