""" Vindby
    Copyright (C) 2018  Esther Dornhelm

    This program is licensed under an Attribution-NonCommercial 4.0 International (CC BY-NC 4.0) license

    Terms and conditions of this license are defined in
    file 'LICENSE.txt', which is part of this source code package.

    See <https://creativecommons.org/licenses/by-nc/4.0/>."""

import numpy

class market():
    """Defines Feed-In Tariff (FIT) selling price for the game, and calculates selling price for a farm"""

    def __init__(self):
        self.turbineRange = [] #Range of turbine sizes in game. used to linearly interpolate feed in tariffs
        self.windRange = [] #Range of mean wind speeds in game. used to linearly interpolate feed in tariffs
        self.capacityRange = [] #Range of capacities in game ,defined manualy in setup.
        self.FITRange = [100, 150]
        self.FIT = {}
        #add something that buys energy, to account for supply and demand balance

    def setup(self, seagrid, game_turbines): #Get min and max values for FIT factors from the game
        turbine_sizes=()
        for t in game_turbines.allTurbines.turbineList:
            turbine_sizes+=(t.cap,)
        self.turbineRange.append(min(turbine_sizes))
        self.turbineRange.append(max(turbine_sizes))

        wind_speeds=()
        for c in seagrid.cellList:
            wind_speeds+=(c.meanU90,)
        self.windRange.append(max(wind_speeds))
        self.windRange.append(min(wind_speeds))
        self.capacityRange = [500, 10]

    def sellingPrice(self, farm): #Determine selling price by taking sum of scaled factors for a farm
        factors = [farm.myTurbineList[-1].type.cap, farm.myCell.meanU90, farm.installedCapacity] #values that will determine the selling price
        factor_limits = [self.turbineRange, self.windRange, self.capacityRange] #Used in loop below
        number_factors = len(factors) #Number of factors being considered. default three.
        factor_weight = [self.FITRange[0] / number_factors, self.FITRange[1] / number_factors] #How much weight does each factor have?

        FIT_value = []  #What is the actual value for each factor. E.g. default is 33 - 50 euro per factor. with three factors, min FIT is 100, max is 150
        for f in range(number_factors):
            m = (factor_weight[0] - factor_weight[1]) / (factor_limits[f][0] - factor_limits[f][1])
            y = (factors[f] - factor_limits[f][1]) * m + factor_weight[1] #Interpolate to get FIT of farm value.
            if y > factor_weight[1]: #do not go above upper limit for FIT
                y = factor_weight[1]
            elif y < factor_weight[0]: #do not go below lower limit for FIT
                y = factor_weight[0]
            FIT_value.append(y) #add contribution to total FIT

        self.FIT[farm] = round(sum(FIT_value),2) #Selling price for farm

        return self.FIT[farm]

    ###STUBBED OUT - include balancing costs here for other renewables. possibly create new objects for other renewable energy supplies.
    #def balancingCosts(self):
