""" Vindby
    Copyright (C) 2018  Esther Dornhelm

    This program is licensed under an Attribution-NonCommercial 4.0 International (CC BY-NC 4.0) license

    Terms and conditions of this license are defined in
    file 'LICENSE.txt', which is part of this source code package.

    See <https://creativecommons.org/licenses/by-nc/4.0/>."""

import math

class Report():
    """Represents a report issued by the game"""

    def __init__(self,game,type):
        self.g=game
        self.issueTime=None
        self.type=type
        self.intervals={}
        self.goal_items = [(self.g.wallet.profits - self.g.wallet.operatingCosts) / self.g.wallet.initialInvestment,self.g.tool.scores['Total'],self.g.energyDemand.renewableShare * 100,self.g.co2.temp, 0]

    def regularReports(self):
        self.intervals['Quinquennial'] = 525600*5
        self.intervals['Annual']=525600
        self.intervals['Monthly'] = 525600/12

    def roundTo(self,value,nearest):
        return int(math.ceil(value / nearest)) * nearest

    def issue(self):
        goal_item = self.goal_items[self.g.myGoal['index']]
        print('\n_______________{} REPORT id {:3.0f} _______________'.format(self.type, self.g.gameClock))#self.issueTime))
        print('Game goal: ',self.g.myGoal['Description'])
        print('Goal progress: {} : {}'.format(self.g.myGoal['measure'],goal_item))
        print('\n>> Basic info:')
        print('\tGame time is: {} '.format(self.g.tool.convertTime(self.g.windClock)['Date']))
        print('\tBalance:        \t\t\t\t €{:,}'.format(self.roundTo(self.g.wallet.checkbalance(),100000)))
        print('\tCapital costs:  \t\t\t\t €{:,}'.format(self.roundTo(self.g.wallet.capitalCosts, 100000)))
        print('\tOperating costs:\t\t\t\t €{:,}'.format(self.roundTo(self.g.wallet.operatingCosts, 100000)))
        print('\tRevenue to date:\t\t\t\t €{:,}'.format(self.roundTo(self.g.wallet.profits, 100000)))

        print('\tInstalled Offshore Wind Capacity: {} MW'.format(self.g.windFarmGrid.installedCapacity))
        if self.g.windFarmGrid.connections!={}:
            print('>> Grid Connections:')
            self.g.windFarmGrid.displayConnections()
        else:
            print('>> Grid Connections: None')
        if self.g.interact.investigatedCells != []:
            print('>> Investigation reports:')
            for cell in self.g.interact.investigatedCells:
                print('\t',cell)
        else:
            print('>> Investigation reports: None')
        print('>> Wind Farm Grid:')
        for farm in self.g.windFarmGrid.windFarmList:
            farm.display()
        for farm in self.g.windFarmGrid.retiredWindFarms:
            farm.display()

        print('>> Construction activities:')
        i=0
        for farm in self.g.windFarmGrid.windFarmList:
            if farm.status['phase']=='Under Construction':
                print('\t{} estimated {:1.1f} days to completion ({:1.1f} days delayed)'.format(farm.name,(-farm.status['age'])/1440,(farm.constructiontime['actual']-farm.constructiontime['original'])/1440))
                i=1 #Indicates there is at least one const activity to show
        if i==0:
            print('\tNone')
        print('>> Climate Change Report')
        self.g.co2.display()
        print('>> Stakeholder events and reactions')
        self.g.public.display()
        self.g.govt.display()
        self.g.shareholders.display()

