""" Vindby
    Copyright (C) 2018  Esther Dornhelm

    This program is licensed under an Attribution-NonCommercial 4.0 International (CC BY-NC 4.0) license

    Terms and conditions of this license are defined in
    file 'LICENSE.txt', which is part of this source code package.

    See <https://creativecommons.org/licenses/by-nc/4.0/>."""

from mCell import *
from mWeather import *

class SeaGrid:
    """Represents the cells in the sea grid"""

    def __init__(self):
        self.cellList=[]
        ##### self.markov=Markov() # UNCOMMENT To regenerate Markov matrices. right now, being read in by txt file.
        self.windInt=60
        ####### TO IMPROVE EFFICIENCY #######
        # Weather simulations are in one hour resolution. increasing this number can greatly increase game speed. Need to go through all functions to check that the previous one hour interval is not static. For example, the turbines still only run once with sixe ten-minute speeds. a for loop would have to be introduced to make sure this computation is done 24 times in one day if the gamespeed is one day.

    def read(self):
        ##### self.markov.setup() # UNCOMMENT To regenerate Markov matrices. right now, being read in by txt file.
        f = open('tcells.txt', 'r')
        self.cellList=[]
        for line in f:
            line = line.strip()
            columns = line.split()
            source = {}
            source['ID'] = int(columns[0])
            source['x'] = float(columns[1])
            source['y'] = float(columns[2])
            source['A'] = float(columns[3])
            source['d'] = (columns[5])
            source['depth'] = float(columns[4])
            source['dist2shore']=float(columns[6])
            source['env_protection']=columns[7] #Is the cell in an environmentally protected area
            source['shipping_channel'] = columns[8]  # Is the cell in shipping channel
            source['status'] = 'unoccupied' #initially
            source['soil']=columns[9]
            source['wind_ampfactor']=float(columns[10])
            cellWeather=Weather(source['wind_ampfactor']) #Each cell has a distinct weather model
            cellWeather.setup()
            x = Cell(source,cellWeather)  # make a new cell from the dictionary
            self.cellList.append(x)
        f.close()

    def occupy(self,cellID,farm):        #Label a cell as occupied when a wind farm is built there
        cellID.status='occupied'
        cellID.myFarm=farm

    def unoccupy(self,cellID):      #Remove label when the wind farm has been removed. This way, the wind farm retains all information about its past (cell, num turbines, etc.) but does not play a role in the game
        cellID.status='unoccupied'


#x=SeaGrid()
#x.read()
#x.display()
#print(x.cellList)
#x.occupy(1)
