""" Vindby
    Copyright (C) 2018  Esther Dornhelm

    This program is licensed under an Attribution-NonCommercial 4.0 International (CC BY-NC 4.0) license

    Terms and conditions of this license are defined in
    file 'LICENSE.txt', which is part of this source code package.

    See <https://creativecommons.org/licenses/by-nc/4.0/>."""

import random

class Substructure:
    """Represents one Substructure"""

    def __init__(self, type, n):
        self.id=n
        self.type=type
        self.name=self.type.type+' '+str(n)
        self.myStructureFailureRates = {}
        self.notification={}
        self.structureFailure = None
        #STUBBED OUT - ADD"self.myFoundation" AND CREATE FOUNDATION OBJECTS: PILES, SUCTION BUCKETS FOR BOTTOM FIXED, AND ANCHOR, PILE, SUCTION BUCKET FOR FLOATING. INTEGRATE INTO COST

    def setup(self, farm, structureFailureRates, turbinesize, soiltype, waveheight, scourprotection):
        for FailType, rate in structureFailureRates.items():         #Adjust failure rates for this structure based on type and design parameters
            self.myStructureFailureRates[FailType]=rate

        if scourprotection == 'None':
            if farm.myCell.soilqual == 'Poor' or farm.myCell.soilqual == 'Med':
                if self.type.type == 'monopile' or self.type.type == 'gravity':
                    self.myStructureFailureRates['structure_localscour'] *= 1
                else:
                    self.myStructureFailureRates['structure_localscour'] = 0
            else:
                self.myStructureFailureRates['structure_localscour'] = 0
        else:
            self.myStructureFailureRates['structure_localscour'] = 0

        if self.type.type == 'tripod' or self.type.type == 'gravity':
            self.myStructureFailureRates['structure_direct_fatigue_cracking']*=0.5 #Very stable structures
        elif self.type.type == 'floating semi submersible' or self.type.type == 'floating spar' or self.type.type =='floating tension leg':
            self.myStructureFailureRates['structure_direct_fatigue_cracking'] *= 2

        if self.type.type != 'gravity':
            self.myStructureFailureRates['structure_bearing'] = 0
        else: #if structure is gravity, no poor soil
            if soiltype == 'Poor':
                self.myStructureFailureRates['structure_bearing'] *= 2
            elif soiltype == 'Good': #med is times 1, poor is times 2
                self.myStructureFailureRates['structure_bearing'] = 0

        if self.type.type == 'jacket' or self.type.type == 'tripod':
            self.myStructureFailureRates['structure_direct_corrosion'] *= 2
        elif self.type.type == 'gravity':
            self.myStructureFailureRates['structure_direct_corrosion'] = 0

        self.myStructureFailureRates['structure_direct_fatigue_cracking'] *=  waveheight #Very rough calc of increasing fatigue load for larger turbines
        self.myStructureFailureRates['structure_external_loading'] *= turbinesize  #very rough calc of increasing load with more sever windspeed/wave height
        if self.type.type == 'gravity' or self.type.type == 'tripod':
            self.myStructureFailureRates['structure_external_loading'] *= 0.5 #considered to be very stable structures

    def checkFailure(self):
        for item in self.myStructureFailureRates: #For each failure in possible failures
            r=random.random()
            if r < self.myStructureFailureRates[item] and self.structureFailure ==None:
                self.structureFailure = item  # Used to indicate that this structure has failed!
                self.notification['fail'] = 'new'  # Used so that the game may detect when a new failure has occured - and prompt the user to fix it immediately, or save for later

#x=SubstructureType({'name':'monopile'})
#y=Substructure(x,3)
#print(y.name)