""" Vindby
    Copyright (C) 2018  Esther Dornhelm

    This program is licensed under an Attribution-NonCommercial 4.0 International (CC BY-NC 4.0) license

    Terms and conditions of this license are defined in
    file 'LICENSE.txt', which is part of this source code package.

    See <https://creativecommons.org/licenses/by-nc/4.0/>."""

import random

class Turbine:
    """Represents one turbine"""

    def __init__(self, type, n):
        self.id=n
        self.type = type
        self.name='Turbine '+str(n)
        self.myTurbineFailureRates = {}
        self.power=None
        self.turbineFailure=None
        self.notification={}

    def setup(self, turbineFailureRates):
        for FailType, rate in turbineFailureRates.items():
            self.myTurbineFailureRates[FailType]=rate

    def run(self,u):
        self.power=self.readPowerCurve(u)
        #STUBBED OUT SECTION WHERE ADDITIONAL CONTROL MEASURES MUST BE INTEGRATED IN THE FUTURE
        return self.power

    def checkFailure(self):
        for item in self.myTurbineFailureRates:
            r=random.random()
            if r < self.myTurbineFailureRates[item] and self.turbineFailure ==None: #Turbine can only have one failure at a time
                self.turbineFailure=item        #Used to indicate that this turbine has failed!
                self.notification['fail']='new' #Used so that the game may detect when a new failure has occured - and prompt the user to fix it immediately, or save for later

    def readPowerCurve(self,u): #Moved to a seperate method for the purposes of computing speculated power lost during failures
        if self.turbineFailure!=None or 'autoshutdown' in self.notification:
            power=0
        elif u < self.type.cutIn or u > self.type.cutOut:
            power=0
        elif u > self.type.ratedSpeed:
            power=self.type.cap
        else:
            power=self.type.slope*(u-self.type.cutIn)
        return power
        #STUBBED OUT - IMPLEMENT ADDITION TURBINE STATES HERE

    def productionLoss(self,u): #calculate loss of production - only depends on speed
        if u < self.type.cutIn or u > self.type.cutOut:
            power=0
        elif u > self.type.ratedSpeed:
            power=self.type.cap
        else:
            power=self.type.slope*(u-self.type.cutIn)
        return power

# x=TurbineType({'cap':3},{'RatedPower': 3.0, 'CutIn': 3.0, 'Slope': 0.33, 'RatedSpeed': 12.0, 'CutOut': 23.0})
# y=Turbine(x,1)
# y.checkFailure({'turbine_mechanical_major': 0.05, 'turbine_mechanical_minor': 0.01, 'turbine_electrical_': 0.01, 'turbine_structural_ship_collision': 0.01, 'turbine_structural_extreme_weather': 0.01, 'turbine_structural_construction': 0.01, 'farm_environmental_': 0.01, 'farm_stakeholder_govt': 0.01, 'farm_stakeholder_public': 0.01, 'farm_stakeholder_shareholder': 0.01})
# y.run(10)
# print(y.turbineFailure)

#y.install(x)