""" Vindby
    Copyright (C) 2018  Esther Dornhelm

    This program is licensed under an Attribution-NonCommercial 4.0 International (CC BY-NC 4.0) license

    Terms and conditions of this license are defined in
    file 'LICENSE.txt', which is part of this source code package.

    See <https://creativecommons.org/licenses/by-nc/4.0/>."""

import mOnshoreSubstation, math

class WindFarmGrid:
    """Compilation of all wind farms"""

    def __init__(self):
        self.windFarmList=[]
        self.retiredWindFarms=[]
        self.failureList=[]         #List of failures that have not been repaired yet
        self.failureHistory=[]      #List with a record of all failures that have occured
        self.installedCapacity=0
        self.connections={}
        self.availableConnections=[]
        self.onshoreSubstation = mOnshoreSubstation.onshoreSubstation()
        self.windfarmnames = []

    def add(self,w):
        self.windFarmList.append(w)
        self.availableConnections.append(w)
        self.windfarmnames.append(w.name)

    def displayConnections(self):
        for connectionfrom, connectionto in self.connections.items():
            print('\tIf {} has a grid failure, {} is down'.format(connectionto.name,connectionfrom.name))

    def listFailures(self,farm,loc,failtype):
        failure=[farm, loc, failtype]
        self.failureList.append(failure)
        self.failureHistory.append(failure)

    def display(self):
        print('\n_____________________________WIND FARM GRID_____________________________')
        for farm in self.windFarmList:
            farm.display()
        for retiredfarm in self.retiredWindFarms:
            retiredfarm.display()
        print()

    def displayFailures(self,costs):
        print('\nList of failures:')
        for failure in self.failureList:  # .items():
            if failure[2] != 'construction_accident':
                print('\t({}) {} using {} maintenance: Location {} : Failure {} : Repair Cost: {} : To be repaired within 30 days'.format(self.failureList.index(failure) + 1, failure[0].name, failure[0].OandM.strategy,failure[1].name, failure[2],int(costs[failure[2]])))
        for failure in self.failureList:  # .items():
            if failure[2] == 'construction_accident':
                print('\t({}) {} using {} maintenance: Location {} : Failure {} : Repair Cost: {} : Repair NOT scheduled'.format(self.failureList.index(failure) + 1, failure[0].name, failure[0].OandM.strategy, failure[1].name,failure[2], int(costs[failure[2]])))

    def retireFarm(self,farm):
        del self.windFarmList[self.windFarmList.index(farm)]
        self.retiredWindFarms.append(farm)
        self.installedCapacity -= farm.installedCapacity

